{
----------------------------------------------------------
MAS-CompMaker was used to generate this code
MAS-CompMaker, copyright(c) 2000, Mats Asplund / MAs Prod.
----------------------------------------------------------

Component Name: TTimePick
        Author: Mats Asplund
      Creation: 2000-10-08
       Version: 1.2
   Description: A component for picking time-values.
        Credit:
        E-mail: mats.asplund@telia.com
          Site: http://go.to/masdp
  Legal issues: Copyright (C) 2000 by Mats Asplund


         Usage: This software is provided 'as-is', without any express or
                implied warranty.  In no event will the author be held liable
                for any  damages arising from the use of this software.

                Permission is granted to anyone to use this software for any
                purpose, including commercial applications, and to alter it
                and redistribute it freely, subject to the following
                restrictions:

                1. The origin of this software must not be misrepresented,
                   you must not claim that you wrote the original software.
                   If you use this software in a product, an acknowledgment
                   in the product documentation would be appreciated but is
                   not required.

                2. Altered source versions must be plainly marked as such, and
                   must not be misrepresented as being the original software.

                3. This notice may not be removed or altered from any source
                   distribution.

                4. If you decide to use this software in any of your applications.
                   Send me an EMail address and tell me about it.

Quick Reference:
                TTimePick inherits from TCompoundPanel. (compound.pas needed)

                Key-Properties:
                  Time: string
                  Hour: string
                  Min: string
                  RoundTheClock: boolean


                Key-Events:
                  OnTimeSet: Occurs when the hour- or minute-values are changed.

--------------------------------------------------------------------------------
}
unit TimePick;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, compound, StdCtrls, ExtCtrls, ComCtrls;

type

  TCForm = class(TForm)
    GB: TGroupBox;
    UD1: TUpDown;
    UD2: TUpDown;
    E1: TEdit;
    E2: TEdit;
  end;

  TOnTimeSetEvent = procedure(Sender: TObject; Time1: TTime; Time2, Hour, Min: string) of object;

  TTimePick = class(TCompoundPanel)
  private
    FForm: TCForm;
    FCopyright: string;
    FOnTimeSet: TOnTimeSetEvent;
    PrevUD1Pos: Integer;
    PrevUD2Pos: Integer;
    FRTC: Boolean;
    function GetTime: string;
    function GetHour: string;
    function GetMin: string;
    procedure UD1Changed;
    procedure UD2Changed;
    procedure TimeChanged(Sender: TObject; var AllowChange: Boolean);
    procedure TimeFormat(EBox, Value: Integer);
    procedure SetCop(Value: string);
  { Private declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Form: TCForm read FForm;
  { Private declarations }
  published
    property Time: string read GetTime;
    property Hour: string read GetHour;
    property Min: string read GetMin;
    property RoundTheClock: Boolean read FRTC write FRTC;
    property OnTimeSet: TOnTimeSetEvent read FOnTimeSet write FOnTimeSet;
    property Copyright: string read FCopyright write SetCop;
  { Published declarations }
  end;

var
  CForm: TCForm;

procedure Register;

implementation
{$R *.DFM}

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TTimePick]);
end;

constructor TTimePick.Create(AOwner: TComponent);
var Hour, Min, Sec, MSec: Word;
begin
  inherited Create(AOwner);
  FForm:= MoveControls(TCForm.Create(nil));
  Form.Height:= 52;
  Form.Width:= 117;
  Form.Borderstyle:= bsNone;
  BevelOuter:= bvNone;
  Form.UD1.OnChanging:= TimeChanged;
  Form.UD2.OnChanging:= TimeChanged;
  FRTC:= true;
  PrevUD1Pos:= -1;
  PrevUD2Pos:= -1;
  DecodeTime(Now, Hour, Min, Sec, MSec);
  Form.UD1.Position:= Hour;
  Form.UD2.Position:= Min;
  UD1Changed;
  UD2Changed;
  FCopyright:= 'Copyright (c) 2000, MAs Prod. / Mats Asplund';
end;

destructor TTimePick.Destroy;
begin
  inherited Destroy;
end;

function TTimePick.GetHour: string;
begin
  Result:= Form.E1.Text;
end;

function TTimePick.GetMin: string;
begin
  Result:= Form.E2.Text;
end;

function TTimePick.GetTime: string;
begin
  Result:= Form.E1.Text + ':' + Form.E2.Text;
end;

procedure TTimePick.SetCop(Value: string);
begin
  Exit;
end;

procedure TTimePick.TimeChanged(Sender: TObject; var AllowChange: Boolean);
begin
  if Sender = Form.UD1 then UD1Changed;
  if Sender = Form.UD2 then UD2Changed;
  if Assigned(FOnTimeSet) then
    FOnTimeSet(Self, StrToTime(Form.E1.Text + ':' + Form.E2.Text),
      Form.E1.Text + ':' + Form.E2.Text, Form.E1.Text, Form.E2.Text);
end;

procedure TTimePick.TimeFormat(EBox, Value: Integer);
begin
  if EBox = 1 then
    case Value of
      0: Form.E1.Text:= '00';
      1: Form.E1.Text:= '01';
      2: Form.E1.Text:= '02';
      3: Form.E1.Text:= '03';
      4: Form.E1.Text:= '04';
      5: Form.E1.Text:= '05';
      6: Form.E1.Text:= '06';
      7: Form.E1.Text:= '07';
      8: Form.E1.Text:= '08';
      9: Form.E1.Text:= '09';
    end
  else
    case Form.UD2.Position of
      0: Form.E2.Text:= '00';
      1: Form.E2.Text:= '01';
      2: Form.E2.Text:= '02';
      3: Form.E2.Text:= '03';
      4: Form.E2.Text:= '04';
      5: Form.E2.Text:= '05';
      6: Form.E2.Text:= '06';
      7: Form.E2.Text:= '07';
      8: Form.E2.Text:= '08';
      9: Form.E2.Text:= '09';
    end;
end;

procedure TTimePick.UD1Changed;
begin
  Form.UD1.OnChanging:= nil;
  if FRTC then
  begin
    if (PrevUD1Pos = 23) and (Form.UD1.Position = 23) then
    begin
      PrevUD1Pos:= -1;
      Form.UD1.Position:= 0;
    end;
    if (PrevUD1Pos = 0) and (Form.UD1.Position = 0) then
    begin
      PrevUD1Pos:= -1;
      Form.UD1.Position:= 23;
    end;
  end;
  Form.E1.Text:= IntToStr(Form.UD1.Position);
  TimeFormat(1, Form.UD1.Position);
  PrevUD1Pos:= Form.UD1.Position;
  Form.UD1.OnChanging:= TimeChanged;
end;

procedure TTimePick.UD2Changed;
begin
  Form.UD1.OnChanging:= nil;
  Form.UD2.OnChanging:= nil;
  if FRTC then
  begin
    if (PrevUD2Pos = 59) and (Form.UD2.Position = 59) then
    begin
      PrevUD2Pos:= -1;
      Form.UD2.Position:= 0;
      if StrToInt(Form.E1.Text) = 23 then
        Form.UD1.Position:= 0
      else
        Form.UD1.Position:= StrToInt(Form.E1.Text) + 1;
      Form.E1.Text:= IntToStr(Form.UD1.Position);
      TimeFormat(1, Form.UD1.Position);
    end;
    if (PrevUD2Pos = 0) and (Form.UD2.Position = 0) then
    begin
      PrevUD2Pos:= -1;
      Form.UD2.Position:= 59;
      if StrToInt(Form.E1.Text) = 0 then
        Form.UD1.Position:= 23
      else
        Form.UD1.Position:= StrToInt(Form.E1.Text) - 1;
      Form.E1.Text:= IntToStr(Form.UD1.Position);
      TimeFormat(1, Form.UD1.Position);
    end;
  end;
  Form.E2.Text:= IntToStr(Form.UD2.Position);
  TimeFormat(2, Form.UD1.Position);
  PrevUD2Pos:= Form.UD2.Position;
  Form.UD1.OnChanging:= TimeChanged;
  Form.UD2.OnChanging:= TimeChanged;
end;

end.

